<?php
/**
 * @file
 * 
 * This script prepares the data for the visualization.
 * 
 * It is responsible for:
 * - querying the database for migration-related data
 * - preparing the data for processing
 * - aggregating the data
 * - setting up variables for later use in Google Maps API javascripts
 * 
 */
    
    //set visualization type baves on URL parameter
    $visType = filter_input( INPUT_GET, 'visType', FILTER_SANITIZE_URL );
    $gridSpacing = filter_input( INPUT_GET, 'grid', FILTER_SANITIZE_URL );
    
    //set default if no parameter is provided
    if ($visType === null) {
        $visType = 'all';
    }
    if ($gridSpacing === null) {
        $gridSpacing = 20;
    }
        
    
    //create database query
    $queryExisting = $db->getQuery(true);
    
    //set up query
    $queryExisting
            ->select($db->quoteName(array(
                        'm.id',
                        'a.nachname',
                        'a.vorname',
                        'm.migration_aus_landessprache',
                        'm.migration_nach_landessprache',
                        'm.jahr',
                        'from.lat',
                        'from.long',
                        'to.lat',
                        'to.long'
                        ),
                    array(
                        'id',
                        'nachname',
                        'vorname',
                        'migration_aus_landessprache',
                        'migration_nach_landessprache',
                        'jahr',
                        'fromLat',
                        'fromLong',
                        'toLat',
                        'toLong'
                        )
                    ))
            ->from($db->quoteName('migration', 'm'))
            ->join('LEFT', $db->quoteName('autorinnen', 'a') . ' ON (' . $db->quoteName('m.autorin_id') . ' = ' . $db->quoteName('a.id') . ')')
            ->join('LEFT', $db->quoteName('geolocations', 'from') . ' ON (' . $db->quoteName('m.migration_aus_landessprache') . ' = ' . $db->quoteName('from.name') . ')')
            ->join('LEFT', $db->quoteName('geolocations', 'to') . ' ON (' . $db->quoteName('m.migration_nach_landessprache') . ' = ' . $db->quoteName('to.name') . ')');
            
    //run query and load data
    $db->setQuery($queryExisting);
    $migrations = $db->loadAssocList();
    
    // set correct types for migration fields
    foreach($migrations as &$migration) {
        $migration['fromPosGrid'] = array(-1,-1);
        $migration['toPosGrid'] = array(-1,-1);
        settype($migration['jahr'],"int");
        settype($migration['fromLat'],"float");
        settype($migration['fromLong'],"float");
        settype($migration['toLat'],"float");
        settype($migration['toLong'],"float");
    }
    
    
    
    // helper function definitions
    
    /**
     * Get spatial distance.
     * 
     * Calculate euclidan distance between two points on the map.
     * 
     * @param float $lat1   latitude of the first point
     * @param float $long1  longitude of the first point
     * @param float $lat2   latitude of the second point
     * @param float $long2  longitude  of the second point
     * @return float        the distance between the two points
     */
    function getSpatialDistance($lat1, $long1, $lat2, $long2) {
        $d = sqrt(pow(($lat2-$lat1), 2) + pow($long2-$long1, 2));
        return $d;
    }
    
    /**
     * Get appropriate grid cell.
     * 
     * Determine the grid cell in which a point lies.
     * 
     * @param float $lat     latitude of the point
     * @param float $long    longitude of the point
     * @param grid $grid     the grind in which the point has to be placed
     * @return (int,int)     the row and column index of the appropriate cell
     */
    function getGridPosition($lat, $long, $grid) {
        $i = floor(($lat - $grid['latMin'])/$grid['maxRadiusLat']);
        $j = floor(($long - $grid['longMin'])/$grid['maxRadiusLong']);
        return array($i, $j);
    }
    
    /**
     * Get the closest centroid.
     * 
     * Determine the closest centroid to a point on the map.
     * 
     * @param float $lat    latitude of the point
     * @param float $long   longitude of the point
     * @param grid $grid    the grind in which to look for the centroid
     * @return (int,int)    the row and column index of the centroid in the grid
     */
    function getClosestCentroid($lat, $long, $grid) {
        $gridPosition = getGridPosition($lat, $long, $grid);
        $bestDistance = 99999.0;
        $closestCentroid = array(-1,-1);
        
        for ($k = max($gridPosition[0]-1,0); $k < min($gridPosition[0]+1,$grid['nRows']);$k++){
            for ($m = max($gridPosition[1]-1,0); $m < min($gridPosition[1]+1,$grid['nCols']);$m++){
                $currentDistance = getSpatialDistance($lat, $long,$grid['centroids'][$k][$m][0],$grid['centroids'][$k][$m][1]);
                if ( $currentDistance <= $bestDistance){
                    $closestCentroid = array($k,$m);
                    $bestDistance = $currentDistance;
                }
            }
        }
  
        return $closestCentroid;
    }
    
    //calculate geolocation data boundaries
    $latMin = 300.0;
    $longMin = 300.0;
    $latMax = -300.0;
    $longMax = -300.0;
           
    foreach ($migrations as $migration) {
        if ($migration['fromLat'] != 999 && $migration['toLat'] != 999){
        
            if($migration['fromLat'] < $latMin)
                $latMin =  $migration['fromLat'];
            if($migration['toLat'] < $latMin)
                $latMin =  $migration['toLat'];
        
            if($migration['fromLat'] > $latMax)
                $latMax =  $migration['fromLat'];
            if($migration['toLat'] > $latMax)
                $latMax =  $migration['toLat'];
        
        
            if($migration['fromLong'] < $longMin)
                $longMin =  $migration['fromLong'];
            if($migration['toLong'] < $longMin)
                $longMin =  $migration['toLong'];
        
            if($migration['fromLong'] > $longMax)
                $longMax =  $migration['fromLong'];
            if($migration['toLong'] > $longMax)
                $longMax =  $migration['toLong'];
        }
    }
    
if ($visType === 'aggregated') {
    //setup grid
    $maxRadiusLat = ($latMax - $latMin) * 1/$gridSpacing;
    $maxRadiusLong = ($longMax - $longMin) * 1/$gridSpacing;
    
    //grid bounds
    $grid['latMin'] = $latMin;
    $grid['latMax'] = $latMax;
    $grid['longMin'] = $longMin;
    $grid['longMax'] = $longMax;
    //grid granularity
    $grid['maxRadiusLat'] = $maxRadiusLat;
    $grid['maxRadiusLong'] = $maxRadiusLong;
    $grid['nRows'] = $gridSpacing;
    $grid['nCols'] = $gridSpacing;
    
    //grid bins
    for ($i = 0; $i<$grid['nRows'];$i++){
        for ($j = 0; $j<$grid['nRows'];$j++){
            $grid['centroids'][$i][$j] = array($latMin + $maxRadiusLat * $i + $maxRadiusLat * 0.5,
                                             $longMin + $maxRadiusLong * $j + $maxRadiusLong * 0.5);
            $grid['aggregatedPositions'][$i][$j] = array(0.0,0.0);
            $grid['populations'][$i][$j] = 0;
        }
    }
    
            
    // calculate closest centroid for every migration route (start and destination)    
    foreach ($migrations as &$migration) {
          
        
        if ($migration['fromLat'] != 999 && $migration['toLat'] != 999) {
            // migration from -->
            $closestCentroidFrom = getClosestCentroid($migration['fromLat'], $migration['fromLong'], $grid);
            
            //update grid
            $grid['populations'][$closestCentroidFrom[0]][$closestCentroidFrom[1]] += 1;
            $grid['aggregatedPositions'][$closestCentroidFrom[0]][$closestCentroidFrom[1]][0] += $migration['fromLat'];
            $grid['aggregatedPositions'][$closestCentroidFrom[0]][$closestCentroidFrom[1]][1] += $migration['fromLong'];
        
            //set assigned cell for migration start
            $migration['fromPosGrid'] = array($closestCentroidFrom[0],$closestCentroidFrom[1]);
        
            //migration to -->
            $closestCentroidTo = getClosestCentroid($migration['toLat'], $migration['toLong'], $grid);
            
            //update grid
            $grid['populations'][$closestCentroidTo[0]][$closestCentroidTo[1]] += 1;
            $grid['aggregatedPositions'][$closestCentroidTo[0]][$closestCentroidTo[1]][0] += $migration['toLat'];
            $grid['aggregatedPositions'][$closestCentroidTo[0]][$closestCentroidTo[1]][1] += $migration['toLong'];
            
            //set assigned cell for migration end
            $migration['toPosGrid'] = array($closestCentroidTo[0],$closestCentroidTo[1]);
        }
    }
    
    
    $maxPopulation = -1;
    
    // recalculate every centroid based on assigned migration route start and end location
    for ($i = 0; $i<$grid['nRows'];$i++){
        for ($j = 0; $j<$grid['nRows'];$j++){
            
            if ($grid['populations'][$i][$j] > $maxPopulation){
                //store maximum population value for later polyline stroke weight calculation
                $maxPopulation = $grid['populations'][$i][$j];
            }
            
            if ($grid['populations'][$i][$j] != 0){
                //calculate new centroid
                $grid['centroids'][$i][$j] =
                    array($grid['aggregatedPositions'][$i][$j][0]/$grid['populations'][$i][$j],
                          $grid['aggregatedPositions'][$i][$j][1]/$grid['populations'][$i][$j]);
            }
        }
    }
}
    

    
